@@ -32,7 +32,7 @@
  * may use your version of this file under either the MPL or the
  * GPL.
  *
- * $Id: nss.h,v 1.26 2003/04/02 23:27:01 wtc%netscape.com Exp $
+ * $Id: nss.h,v 1.34 2004/01/08 01:04:56 wchang0222%aol.com Exp $
  */
 
 #ifndef __nss_h_
@@ -49,9 +49,9 @@ SEC_BEGIN_PROTOS
  * The format of the version string should be
  *     "<major version>.<minor version>[.<patch level>] [<Beta>]"
  */
-#define NSS_VERSION  "3.8"
+#define NSS_VERSION  "3.9"
 #define NSS_VMAJOR   3
-#define NSS_VMINOR   8
+#define NSS_VMINOR   9
 #define NSS_VPATCH   0
 #define NSS_BETA     PR_FALSE
 
@@ -122,6 +122,12 @@ extern SECStatus NSS_InitReadWrite(const char *configdir);
 #define NSS_INIT_NOROOTINIT     0x10
 #define NSS_INIT_OPTIMIZESPACE  0x20
 
+#ifdef macintosh
+#define SECMOD_DB "Security Modules"
+#else
+#define SECMOD_DB "secmod.db"
+#endif
+
 extern SECStatus NSS_Initialize(const char *configdir, 
 	const char *certPrefix, const char *keyPrefix, 
 	const char *secmodName, PRUint32 flags);
