@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: devmod.c,v $ $Revision: 1.3 $ $Date: 2002/04/04 20:00:21 $ $Name: NSS_3_8_RTM $";
+static const char CVS_ID[] = "@(#) $RCSfile: devmod.c,v $ $Revision: 1.5 $ $Date: 2003/08/01 02:02:43 $ $Name: NSS_3_9_RTM $";
 #endif /* DEBUG */
 
 #ifndef NSSCKEPV_H
@@ -205,8 +205,7 @@ loser:
 }
 
 NSS_IMPLEMENT PRStatus
-nssModule_Load
-(
+nssModule_Load (
   NSSModule *mod
 )
 {
@@ -296,8 +295,7 @@ loser:
 }
 
 NSS_IMPLEMENT PRStatus
-nssModule_Unload
-(
+nssModule_Unload (
   NSSModule *mod
 )
 {
@@ -317,8 +315,7 @@ nssModule_Unload
  *  if provided.  XXX use the opaque arg also, right? 
  */
 NSS_IMPLEMENT NSSModule *
-nssModule_Create
-(
+nssModule_Create (
   NSSUTF8 *moduleOpt,
   NSSUTF8 *uriOpt,
   NSSUTF8 *opaqueOpt,
@@ -367,8 +364,7 @@ loser:
 }
 
 NSS_EXTERN PRStatus
-nssCryptokiArgs_ParseNextPair
-(
+nssCryptokiArgs_ParseNextPair (
   NSSUTF8 *start,
   NSSUTF8 **attrib,
   NSSUTF8 **value,
@@ -377,8 +373,7 @@ nssCryptokiArgs_ParseNextPair
 );
 
 static PRStatus
-parse_slot_flags
-(
+parse_slot_flags (
   NSSSlot *slot,
   NSSUTF8 *slotFlags
 )
@@ -433,8 +428,7 @@ parse_slot_flags
 }
 
 static PRStatus
-parse_slot_parameters
-(
+parse_slot_parameters (
   NSSSlot *slot,
   NSSUTF8 *slotParams,
   NSSArena *tmparena
@@ -473,8 +467,7 @@ get_slot_number(NSSUTF8* snString)
 }
 
 static PRStatus
-parse_module_slot_parameters
-(
+parse_module_slot_parameters (
   NSSModule *mod,
   NSSUTF8 *slotParams
 )
@@ -510,8 +503,7 @@ parse_module_slot_parameters
 }
 
 static PRStatus
-parse_nss_flags
-(
+parse_nss_flags (
   NSSModule *mod,
   NSSUTF8 *nssFlags
 )
@@ -542,8 +534,7 @@ parse_nss_flags
 }
 
 static PRStatus
-parse_nss_parameters
-(
+parse_nss_parameters (
   NSSModule *mod,
   NSSUTF8 *nssParams,
   NSSArena *tmparena,
@@ -580,8 +571,7 @@ parse_nss_parameters
 }
 
 static PRStatus
-parse_module_parameters
-(
+parse_module_parameters (
   NSSModule *mod,
   NSSUTF8 *moduleParams,
   NSSUTF8 **slotParams
@@ -621,8 +611,7 @@ parse_module_parameters
 }
 
 static NSSUTF8 **
-get_module_specs
-(
+get_module_specs (
   NSSModule *mod
 )
 {
@@ -637,8 +626,7 @@ get_module_specs
 
 /* XXX continue working on */
 NSS_IMPLEMENT NSSModule *
-nssModule_CreateFromSpec
-(
+nssModule_CreateFromSpec (
   NSSUTF8 *moduleSpec,
   NSSModule *parent,
   PRBool loadSubModules
@@ -721,14 +709,12 @@ loser:
 }
 
 NSS_IMPLEMENT PRStatus
-nssModule_Destroy
-(
+nssModule_Destroy (
   NSSModule *mod
 )
 {
     PRUint32 i, numSlots;
-    PR_AtomicDecrement(&mod->base.refCount);
-    if (mod->base.refCount == 0) {
+    if (PR_AtomicDecrement(&mod->base.refCount) == 0) {
 	if (mod->numSlots == 0) {
 	    (void)nssModule_Unload(mod);
 	    return nssArena_Destroy(mod->base.arena);
@@ -743,8 +729,7 @@ nssModule_Destroy
 }
 
 NSS_IMPLEMENT PRStatus
-nssModule_DestroyFromSlot
-(
+nssModule_DestroyFromSlot (
   NSSModule *mod,
   NSSSlot *slot
 )
@@ -766,8 +751,7 @@ nssModule_DestroyFromSlot
 }
 
 NSS_IMPLEMENT NSSModule *
-nssModule_AddRef
-(
+nssModule_AddRef (
   NSSModule *mod
 )
 {
@@ -776,8 +760,7 @@ nssModule_AddRef
 }
 
 NSS_IMPLEMENT NSSUTF8 *
-nssModule_GetName
-(
+nssModule_GetName (
   NSSModule *mod
 )
 {
@@ -785,8 +768,7 @@ nssModule_GetName
 }
 
 NSS_IMPLEMENT PRBool
-nssModule_IsThreadSafe
-(
+nssModule_IsThreadSafe (
   NSSModule *module
 )
 {
@@ -794,8 +776,7 @@ nssModule_IsThreadSafe
 }
 
 NSS_IMPLEMENT PRBool
-nssModule_IsInternal
-(
+nssModule_IsInternal (
   NSSModule *mod
 )
 {
@@ -803,8 +784,7 @@ nssModule_IsInternal
 }
 
 NSS_IMPLEMENT PRBool
-nssModule_IsModuleDBOnly
-(
+nssModule_IsModuleDBOnly (
   NSSModule *mod
 )
 {
@@ -812,8 +792,7 @@ nssModule_IsModuleDBOnly
 }
 
 NSS_IMPLEMENT void *
-nssModule_GetCryptokiEPV
-(
+nssModule_GetCryptokiEPV (
   NSSModule *mod
 )
 {
@@ -821,8 +800,7 @@ nssModule_GetCryptokiEPV
 }
 
 NSS_IMPLEMENT NSSSlot **
-nssModule_GetSlots
-(
+nssModule_GetSlots (
   NSSModule *mod
 )
 {
@@ -838,8 +816,7 @@ nssModule_GetSlots
 }
 
 NSS_IMPLEMENT NSSSlot *
-nssModule_FindSlotByName
-(
+nssModule_FindSlotByName (
   NSSModule *mod,
   NSSUTF8 *slotName
 )
@@ -862,8 +839,7 @@ nssModule_FindSlotByName
 }
 
 NSS_IMPLEMENT NSSToken *
-nssModule_FindTokenByName
-(
+nssModule_FindTokenByName (
   NSSModule *mod,
   NSSUTF8 *tokenName
 )
@@ -888,8 +864,7 @@ nssModule_FindTokenByName
 }
 
 NSS_IMPLEMENT PRInt32
-nssModule_GetCertOrder
-(
+nssModule_GetCertOrder (
   NSSModule *module
 )
 {
