@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: devslot.c,v $ $Revision: 1.12 $ $Date: 2002/09/06 00:27:37 $ $Name: NSS_3_8_RTM $";
+static const char CVS_ID[] = "@(#) $RCSfile: devslot.c,v $ $Revision: 1.18 $ $Date: 2003/08/01 02:02:43 $ $Name: NSS_3_9_RTM $";
 #endif /* DEBUG */
 
 #ifndef NSSCKEPV_H
@@ -76,6 +76,7 @@ struct NSSSlotStr
   CK_FLAGS ckFlags; /* from CK_SLOT_INFO.flags */
   struct nssSlotAuthInfoStr authInfo;
   PRIntervalTime lastTokenPing;
+  PZLock *lock;
 #ifdef NSS_3_4_CODE
   PK11SlotInfo *pk11slot;
 #endif
@@ -94,8 +95,7 @@ static const CK_FLAGS s_ck_readonly_flags = CKF_SERIAL_SESSION;
 #ifdef PURE_STAN_BUILD
 /* In pk11slot.c, this was a no-op.  So it is here also. */
 static CK_RV PR_CALLBACK
-nss_ck_slot_notify
-(
+nss_ck_slot_notify (
   CK_SESSION_HANDLE session,
   CK_NOTIFICATION event,
   CK_VOID_PTR pData
@@ -105,8 +105,7 @@ nss_ck_slot_notify
 }
 
 NSS_IMPLEMENT NSSSlot *
-nssSlot_Create
-(
+nssSlot_Create (
   CK_SLOT_ID slotID,
   NSSModule *parent
 )
@@ -151,6 +150,9 @@ nssSlot_Create
     if (!rvSlot->base.lock) {
 	goto loser;
     }
+    if (!nssModule_IsThreadSafe(parent)) {
+	rvSlot->lock = nssModule_GetLock(parent);
+    }
     rvSlot->module = parent; /* refs go from module to slots */
     rvSlot->slotID = slotID;
     rvSlot->ckFlags = slotInfo.flags;
@@ -171,14 +173,12 @@ loser:
 #endif /* PURE_STAN_BUILD */
 
 NSS_IMPLEMENT PRStatus
-nssSlot_Destroy
-(
+nssSlot_Destroy (
   NSSSlot *slot
 )
 {
     if (slot) {
-	PR_AtomicDecrement(&slot->base.refCount);
-	if (slot->base.refCount == 0) {
+	if (PR_AtomicDecrement(&slot->base.refCount) == 0) {
 	    PZ_DestroyLock(slot->base.lock);
 #ifdef PURE_STAN_BUILD
 	    nssToken_Destroy(slot->token);
@@ -190,9 +190,24 @@ nssSlot_Destroy
     return PR_SUCCESS;
 }
 
+void
+nssSlot_EnterMonitor(NSSSlot *slot)
+{
+    if (slot->lock) {
+	PZ_Lock(slot->lock);
+    }
+}
+
+void
+nssSlot_ExitMonitor(NSSSlot *slot)
+{
+    if (slot->lock) {
+	PZ_Unlock(slot->lock);
+    }
+}
+
 NSS_IMPLEMENT void
-NSSSlot_Destroy
-(
+NSSSlot_Destroy (
   NSSSlot *slot
 )
 {
@@ -200,8 +215,7 @@ NSSSlot_Destroy
 }
 
 NSS_IMPLEMENT NSSSlot *
-nssSlot_AddRef
-(
+nssSlot_AddRef (
   NSSSlot *slot
 )
 {
@@ -210,8 +224,7 @@ nssSlot_AddRef
 }
 
 NSS_IMPLEMENT NSSUTF8 *
-nssSlot_GetName
-(
+nssSlot_GetName (
   NSSSlot *slot
 )
 {
@@ -219,8 +232,7 @@ nssSlot_GetName
 }
 
 NSS_IMPLEMENT NSSUTF8 *
-nssSlot_GetTokenName
-(
+nssSlot_GetTokenName (
   NSSSlot *slot
 )
 {
@@ -237,8 +249,7 @@ within_token_delay_period(NSSSlot *slot)
     }
     time = PR_IntervalNow();
     lastTime = slot->lastTokenPing;
-    if ((lastTime) &&
-	(time > lastTime) && ((time - lastTime) < s_token_delay_time)) {
+    if ((lastTime) && ((time - lastTime) < s_token_delay_time)) {
 	return PR_TRUE;
     }
     slot->lastTokenPing = time;
@@ -246,8 +257,7 @@ within_token_delay_period(NSSSlot *slot)
 }
 
 NSS_IMPLEMENT PRBool
-nssSlot_IsTokenPresent
-(
+nssSlot_IsTokenPresent (
   NSSSlot *slot
 )
 {
@@ -265,6 +275,7 @@ nssSlot_IsTokenPresent
     if (within_token_delay_period(slot)) {
 	return (PRBool)((slot->ckFlags & CKF_TOKEN_PRESENT) != 0);
     }
+
     /* First obtain the slot info */
 #ifdef PURE_STAN_BUILD
     epv = nssModule_GetCryptokiEPV(slot->module);
@@ -274,7 +285,9 @@ nssSlot_IsTokenPresent
     if (!epv) {
 	return PR_FALSE;
     }
+    nssSlot_EnterMonitor(slot);
     ckrv = CKAPI(epv)->C_GetSlotInfo(slot->slotID, &slotInfo);
+    nssSlot_ExitMonitor(slot);
     if (ckrv != CKR_OK) {
 	slot->token->base.name[0] = 0; /* XXX */
 	return PR_FALSE;
@@ -350,8 +363,7 @@ nssSlot_IsTokenPresent
 
 #ifdef PURE_STAN_BUILD
 NSS_IMPLEMENT NSSModule *
-nssSlot_GetModule
-(
+nssSlot_GetModule (
   NSSSlot *slot
 )
 {
@@ -360,8 +372,7 @@ nssSlot_GetModule
 #endif /* PURE_STAN_BUILD */
 
 NSS_IMPLEMENT void *
-nssSlot_GetCryptokiEPV
-(
+nssSlot_GetCryptokiEPV (
   NSSSlot *slot
 )
 {
@@ -373,8 +384,7 @@ nssSlot_GetCryptokiEPV
 }
 
 NSS_IMPLEMENT NSSToken *
-nssSlot_GetToken
-(
+nssSlot_GetToken (
   NSSSlot *slot
 )
 {
@@ -386,8 +396,7 @@ nssSlot_GetToken
 
 #ifdef PURE_STAN_BUILD
 NSS_IMPLEMENT PRBool
-nssSlot_IsPermanent
-(
+nssSlot_IsPermanent (
   NSSSlot *slot
 )
 {
@@ -395,8 +404,7 @@ nssSlot_IsPermanent
 }
 
 NSS_IMPLEMENT PRBool
-nssSlot_IsFriendly
-(
+nssSlot_IsFriendly (
   NSSSlot *slot
 )
 {
@@ -404,8 +412,7 @@ nssSlot_IsFriendly
 }
 
 NSS_IMPLEMENT PRBool
-nssSlot_IsHardware
-(
+nssSlot_IsHardware (
   NSSSlot *slot
 )
 {
@@ -413,8 +420,7 @@ nssSlot_IsHardware
 }
 
 NSS_IMPLEMENT PRStatus
-nssSlot_Refresh
-(
+nssSlot_Refresh (
   NSSSlot *slot
 )
 {
@@ -429,8 +435,7 @@ nssSlot_Refresh
 }
 
 static PRBool
-slot_needs_login
-(
+slot_needs_login (
   NSSSlot *slot,
   nssSession *session
 )
@@ -486,8 +491,7 @@ slot_needs_login
 }
 
 static PRStatus
-slot_login
-(
+slot_login (
   NSSSlot *slot, 
   nssSession *session, 
   CK_USER_TYPE userType, 
@@ -549,8 +553,7 @@ slot_login
 }
 
 static PRStatus
-init_slot_password
-(
+init_slot_password (
   NSSSlot *slot, 
   nssSession *rwSession, 
   NSSUTF8 *password
@@ -590,8 +593,7 @@ loser:
 }
 
 static PRStatus
-change_slot_password
-(
+change_slot_password (
   NSSSlot *slot, 
   nssSession *rwSession, 
   NSSUTF8 *oldPassword,
@@ -632,8 +634,7 @@ change_slot_password
 }
 
 NSS_IMPLEMENT PRStatus
-nssSlot_Login
-(
+nssSlot_Login (
   NSSSlot *slot,
   NSSCallback *pwcb
 )
@@ -672,8 +673,7 @@ nssSlot_Login
 }
 
 NSS_IMPLEMENT PRStatus
-nssSlot_Logout
-(
+nssSlot_Logout (
   NSSSlot *slot,
   nssSession *sessionOpt
 )
@@ -696,8 +696,7 @@ nssSlot_Logout
 }
 
 NSS_IMPLEMENT PRBool
-nssSlot_IsLoggedIn
-(
+nssSlot_IsLoggedIn (
   NSSSlot *slot
 )
 {
@@ -706,8 +705,7 @@ nssSlot_IsLoggedIn
 }
 
 NSS_IMPLEMENT void
-nssSlot_SetPasswordDefaults
-(
+nssSlot_SetPasswordDefaults (
   NSSSlot *slot,
   PRInt32 askPasswordTimeout
 )
@@ -717,8 +715,7 @@ nssSlot_SetPasswordDefaults
 
 
 NSS_IMPLEMENT PRStatus
-nssSlot_SetPassword
-(
+nssSlot_SetPassword (
   NSSSlot *slot,
   NSSUTF8 *oldPasswordOpt,
   NSSUTF8 *newPassword
@@ -746,8 +743,7 @@ nssSlot_SetPassword
 }
 
 NSS_IMPLEMENT nssSession *
-nssSlot_CreateSession
-(
+nssSlot_CreateSession (
   NSSSlot *slot,
   NSSArena *arenaOpt,
   PRBool readWrite /* so far, this is the only flag used */
@@ -772,9 +768,9 @@ nssSlot_CreateSession
     if (!rvSession) {
 	return (nssSession *)NULL;
     }
-    if (!nssModule_IsThreadSafe(slot->module)) {
-	/* If the parent module is not threadsafe, create lock to manage 
-	 * session within threads.
+    if (nssModule_IsThreadSafe(slot->module)) {
+	/* If the parent module is threadsafe, 
+         * create lock to protect just this session.
 	 */
 	rvSession->lock = PZ_NewLock(nssILockOther);
 	if (!rvSession->lock) {
@@ -785,7 +781,12 @@ nssSlot_CreateSession
 	    }
 	    return (nssSession *)NULL;
 	}
+        rvSession->ownLock = PR_TRUE;
+    } else {
+        rvSession->lock = slot->lock;
+        rvSession->ownLock = PR_FALSE;
     }
+	
     rvSession->handle = handle;
     rvSession->slot = slot;
     rvSession->isRW = readWrite;
@@ -793,8 +794,7 @@ nssSlot_CreateSession
 }
 
 NSS_IMPLEMENT PRStatus
-nssSession_Destroy
-(
+nssSession_Destroy (
   nssSession *s
 )
 {
@@ -802,7 +802,7 @@ nssSession_Destroy
     if (s) {
 	void *epv = s->slot->epv;
 	ckrv = CKAPI(epv)->C_CloseSession(s->handle);
-	if (s->lock) {
+	if (s->ownLock && s->lock) {
 	    PZ_DestroyLock(s->lock);
 	}
 	nss_ZFreeIf(s);
@@ -812,8 +812,7 @@ nssSession_Destroy
 #endif /* PURE_STAN_BUILD */
 
 NSS_IMPLEMENT PRStatus
-nssSession_EnterMonitor
-(
+nssSession_EnterMonitor (
   nssSession *s
 )
 {
@@ -822,8 +821,7 @@ nssSession_EnterMonitor
 }
 
 NSS_IMPLEMENT PRStatus
-nssSession_ExitMonitor
-(
+nssSession_ExitMonitor (
   nssSession *s
 )
 {
@@ -831,8 +829,7 @@ nssSession_ExitMonitor
 }
 
 NSS_EXTERN PRBool
-nssSession_IsReadWrite
-(
+nssSession_IsReadWrite (
   nssSession *s
 )
 {
