@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: devtoken.c,v $ $Revision: 1.33 $ $Date: 2003/01/24 06:37:03 $ $Name: NSS_3_8_RTM $";
+static const char CVS_ID[] = "@(#) $RCSfile: devtoken.c,v $ $Revision: 1.36 $ $Date: 2003/09/23 20:34:15 $ $Name: NSS_3_9_RTM $";
 #endif /* DEBUG */
 
 #ifndef NSSCKEPV_H
@@ -69,8 +69,7 @@ struct NSSTokenStr
 };
 
 NSS_IMPLEMENT NSSToken *
-nssToken_Create
-(
+nssToken_Create (
   CK_SLOT_ID slotID,
   NSSSlot *peer
 )
@@ -148,14 +147,12 @@ loser:
 #endif /* PURE_STAN_BUILD */
 
 NSS_IMPLEMENT PRStatus
-nssToken_Destroy
-(
+nssToken_Destroy (
   NSSToken *tok
 )
 {
     if (tok) {
-	PR_AtomicDecrement(&tok->base.refCount);
-	if (tok->base.refCount == 0) {
+	if (PR_AtomicDecrement(&tok->base.refCount) == 0) {
 	    PZ_DestroyLock(tok->base.lock);
 	    nssTokenObjectCache_Destroy(tok->cache);
 	    return nssArena_Destroy(tok->base.arena);
@@ -165,8 +162,7 @@ nssToken_Destroy
 }
 
 NSS_IMPLEMENT void
-nssToken_Remove
-(
+nssToken_Remove (
   NSSToken *tok
 )
 {
@@ -174,8 +170,7 @@ nssToken_Remove
 }
 
 NSS_IMPLEMENT void
-NSSToken_Destroy
-(
+NSSToken_Destroy (
   NSSToken *tok
 )
 {
@@ -183,8 +178,7 @@ NSSToken_Destroy
 }
 
 NSS_IMPLEMENT NSSToken *
-nssToken_AddRef
-(
+nssToken_AddRef (
   NSSToken *tok
 )
 {
@@ -193,8 +187,7 @@ nssToken_AddRef
 }
 
 NSS_IMPLEMENT NSSSlot *
-nssToken_GetSlot
-(
+nssToken_GetSlot (
   NSSToken *tok
 )
 {
@@ -203,8 +196,7 @@ nssToken_GetSlot
 
 #ifdef PURE_STAN_BUILD
 NSS_IMPLEMENT NSSModule *
-nssToken_GetModule
-(
+nssToken_GetModule (
   NSSToken *token
 )
 {
@@ -213,8 +205,7 @@ nssToken_GetModule
 #endif
 
 NSS_IMPLEMENT void *
-nssToken_GetCryptokiEPV
-(
+nssToken_GetCryptokiEPV (
   NSSToken *token
 )
 {
@@ -222,8 +213,7 @@ nssToken_GetCryptokiEPV
 }
 
 NSS_IMPLEMENT nssSession *
-nssToken_GetDefaultSession
-(
+nssToken_GetDefaultSession (
   NSSToken *token
 )
 {
@@ -231,8 +221,7 @@ nssToken_GetDefaultSession
 }
 
 NSS_IMPLEMENT NSSUTF8 *
-nssToken_GetName
-(
+nssToken_GetName (
   NSSToken *tok
 )
 {
@@ -246,8 +235,7 @@ nssToken_GetName
 }
 
 NSS_IMPLEMENT NSSUTF8 *
-NSSToken_GetName
-(
+NSSToken_GetName (
   NSSToken *token
 )
 {
@@ -255,8 +243,7 @@ NSSToken_GetName
 }
 
 NSS_IMPLEMENT PRBool
-nssToken_IsLoginRequired
-(
+nssToken_IsLoginRequired (
   NSSToken *token
 )
 {
@@ -264,8 +251,7 @@ nssToken_IsLoginRequired
 }
 
 NSS_IMPLEMENT PRBool
-nssToken_NeedsPINInitialization
-(
+nssToken_NeedsPINInitialization (
   NSSToken *token
 )
 {
@@ -273,8 +259,7 @@ nssToken_NeedsPINInitialization
 }
 
 NSS_IMPLEMENT PRStatus
-nssToken_DeleteStoredObject
-(
+nssToken_DeleteStoredObject (
   nssCryptokiObject *instance
 )
 {
@@ -309,8 +294,7 @@ nssToken_DeleteStoredObject
 }
 
 static nssCryptokiObject *
-import_object
-(
+import_object (
   NSSToken *tok,
   nssSession *sessionOpt,
   CK_ATTRIBUTE_PTR objectTemplate,
@@ -357,8 +341,7 @@ import_object
 }
 
 static nssCryptokiObject **
-create_objects_from_handles
-(
+create_objects_from_handles (
   NSSToken *tok,
   nssSession *session,
   CK_OBJECT_HANDLE *handles,
@@ -383,8 +366,7 @@ create_objects_from_handles
 }
 
 static nssCryptokiObject **
-find_objects
-(
+find_objects (
   NSSToken *tok,
   nssSession *sessionOpt,
   CK_ATTRIBUTE_PTR obj_template,
@@ -506,8 +488,7 @@ loser:
 }
 
 static nssCryptokiObject **
-find_objects_by_template
-(
+find_objects_by_template (
   NSSToken *token,
   nssSession *sessionOpt,
   CK_ATTRIBUTE_PTR obj_template,
@@ -530,6 +511,7 @@ find_objects_by_template
 #ifdef NSS_3_4_CODE
 	PORT_SetError(SEC_ERROR_LIBRARY_FAILURE);
 #endif
+	if (statusOpt) *statusOpt = PR_FAILURE;
 	return NULL;
     }
     /* If these objects are being cached, try looking there first */
@@ -558,8 +540,7 @@ find_objects_by_template
 extern const NSSError NSS_ERROR_INVALID_CERTIFICATE;
 
 NSS_IMPLEMENT nssCryptokiObject *
-nssToken_ImportCertificate
-(
+nssToken_ImportCertificate (
   NSSToken *tok,
   nssSession *sessionOpt,
   NSSCertificateType certType,
@@ -683,8 +664,7 @@ nssToken_ImportCertificate
  * has been marked as "traversable"
  */
 NSS_IMPLEMENT nssCryptokiObject **
-nssToken_FindCertificates
-(
+nssToken_FindCertificates (
   NSSToken *token,
   nssSession *sessionOpt,
   nssTokenSearchType searchType,
@@ -720,8 +700,7 @@ nssToken_FindCertificates
 }
 
 NSS_IMPLEMENT nssCryptokiObject **
-nssToken_FindCertificatesBySubject
-(
+nssToken_FindCertificatesBySubject (
   NSSToken *token,
   nssSession *sessionOpt,
   NSSDER *subject,
@@ -752,8 +731,7 @@ nssToken_FindCertificatesBySubject
 }
 
 NSS_IMPLEMENT nssCryptokiObject **
-nssToken_FindCertificatesByNickname
-(
+nssToken_FindCertificatesByNickname (
   NSSToken *token,
   nssSession *sessionOpt,
   NSSUTF8 *name,
@@ -802,8 +780,7 @@ nssToken_FindCertificatesByNickname
  * it just won't return a value for it.
  */
 NSS_IMPLEMENT nssCryptokiObject **
-nssToken_FindCertificatesByEmail
-(
+nssToken_FindCertificatesByEmail (
   NSSToken *token,
   nssSession *sessionOpt,
   NSSASCII7 *email,
@@ -846,8 +823,7 @@ nssToken_FindCertificatesByEmail
 }
 
 NSS_IMPLEMENT nssCryptokiObject **
-nssToken_FindCertificatesByID
-(
+nssToken_FindCertificatesByID (
   NSSToken *token,
   nssSession *sessionOpt,
   NSSItem *id,
@@ -920,8 +896,7 @@ nssToken_decodeSerialItem(NSSItem *serial, NSSItem *serialDecode)
 }
 
 NSS_IMPLEMENT nssCryptokiObject *
-nssToken_FindCertificateByIssuerAndSerialNumber
-(
+nssToken_FindCertificateByIssuerAndSerialNumber (
   NSSToken *token,
   nssSession *sessionOpt,
   NSSDER *issuer,
@@ -996,8 +971,7 @@ nssToken_FindCertificateByIssuerAndSerialNumber
 }
 
 NSS_IMPLEMENT nssCryptokiObject *
-nssToken_FindCertificateByEncodedCertificate
-(
+nssToken_FindCertificateByEncodedCertificate (
   NSSToken *token,
   nssSession *sessionOpt,
   NSSBER *encodedCertificate,
@@ -1032,8 +1006,7 @@ nssToken_FindCertificateByEncodedCertificate
 }
 
 NSS_IMPLEMENT nssCryptokiObject **
-nssToken_FindPrivateKeys
-(
+nssToken_FindPrivateKeys (
   NSSToken *token,
   nssSession *sessionOpt,
   nssTokenSearchType searchType,
@@ -1063,8 +1036,7 @@ nssToken_FindPrivateKeys
 
 /* XXX ?there are no session cert objects, so only search token objects */
 NSS_IMPLEMENT nssCryptokiObject *
-nssToken_FindPrivateKeyByID
-(
+nssToken_FindPrivateKeyByID (
   NSSToken *token,
   nssSession *sessionOpt,
   NSSItem *keyID
@@ -1094,8 +1066,7 @@ nssToken_FindPrivateKeyByID
 
 /* XXX ?there are no session cert objects, so only search token objects */
 NSS_IMPLEMENT nssCryptokiObject *
-nssToken_FindPublicKeyByID
-(
+nssToken_FindPublicKeyByID (
   NSSToken *token,
   nssSession *sessionOpt,
   NSSItem *keyID
@@ -1160,8 +1131,7 @@ md5_hash(NSSItem *input, NSSItem *output)
 }
 
 static CK_TRUST
-get_ck_trust
-(
+get_ck_trust (
   nssTrustLevel nssTrust
 )
 {
@@ -1180,8 +1150,7 @@ get_ck_trust
 }
  
 NSS_IMPLEMENT nssCryptokiObject *
-nssToken_ImportTrust
-(
+nssToken_ImportTrust (
   NSSToken *tok,
   nssSession *sessionOpt,
   NSSDER *certEncoding,
@@ -1238,8 +1207,7 @@ nssToken_ImportTrust
 }
 
 NSS_IMPLEMENT nssCryptokiObject **
-nssToken_FindTrustObjects
-(
+nssToken_FindTrustObjects (
   NSSToken *token,
   nssSession *sessionOpt,
   nssTokenSearchType searchType,
@@ -1277,8 +1245,7 @@ nssToken_FindTrustObjects
 }
 
 NSS_IMPLEMENT nssCryptokiObject *
-nssToken_FindTrustForCertificate
-(
+nssToken_FindTrustForCertificate (
   NSSToken *token,
   nssSession *sessionOpt,
   NSSDER *certEncoding,
@@ -1316,8 +1283,7 @@ nssToken_FindTrustForCertificate
 }
  
 NSS_IMPLEMENT nssCryptokiObject *
-nssToken_ImportCRL
-(
+nssToken_ImportCRL (
   NSSToken *token,
   nssSession *sessionOpt,
   NSSDER *subject,
@@ -1360,8 +1326,7 @@ nssToken_ImportCRL
 }
 
 NSS_IMPLEMENT nssCryptokiObject **
-nssToken_FindCRLs
-(
+nssToken_FindCRLs (
   NSSToken *token,
   nssSession *sessionOpt,
   nssTokenSearchType searchType,
@@ -1399,8 +1364,7 @@ nssToken_FindCRLs
 }
 
 NSS_IMPLEMENT nssCryptokiObject **
-nssToken_FindCRLsBySubject
-(
+nssToken_FindCRLsBySubject (
   NSSToken *token,
   nssSession *sessionOpt,
   NSSDER *subject,
@@ -1434,8 +1398,7 @@ nssToken_FindCRLsBySubject
 }
 
 NSS_IMPLEMENT PRStatus
-nssToken_GetCachedObjectAttributes
-(
+nssToken_GetCachedObjectAttributes (
   NSSToken *token,
   NSSArena *arenaOpt,
   nssCryptokiObject *object,
@@ -1453,8 +1416,7 @@ nssToken_GetCachedObjectAttributes
 }
 
 NSS_IMPLEMENT NSSItem *
-nssToken_Digest
-(
+nssToken_Digest (
   NSSToken *tok,
   nssSession *sessionOpt,
   NSSAlgorithmAndParameters *ap,
@@ -1521,8 +1483,7 @@ nssToken_Digest
 }
 
 NSS_IMPLEMENT PRStatus
-nssToken_BeginDigest
-(
+nssToken_BeginDigest (
   NSSToken *tok,
   nssSession *sessionOpt,
   NSSAlgorithmAndParameters *ap
@@ -1539,8 +1500,7 @@ nssToken_BeginDigest
 }
 
 NSS_IMPLEMENT PRStatus
-nssToken_ContinueDigest
-(
+nssToken_ContinueDigest (
   NSSToken *tok,
   nssSession *sessionOpt,
   NSSItem *item
@@ -1559,8 +1519,7 @@ nssToken_ContinueDigest
 }
 
 NSS_IMPLEMENT NSSItem *
-nssToken_FinishDigest
-(
+nssToken_FinishDigest (
   NSSToken *tok,
   nssSession *sessionOpt,
   NSSItem *rvOpt,
@@ -1612,8 +1571,7 @@ nssToken_FinishDigest
 }
 
 NSS_IMPLEMENT PRBool
-nssToken_IsPresent
-(
+nssToken_IsPresent (
   NSSToken *token
 )
 {
@@ -1630,8 +1588,7 @@ nssToken_IsPresent
  * increasing the likelihood that the cache takes care of it.
  */
 NSS_IMPLEMENT PRStatus
-nssToken_TraverseCertificates
-(
+nssToken_TraverseCertificates (
   NSSToken *token,
   nssSession *sessionOpt,
   nssTokenSearchType searchType,
@@ -1731,8 +1688,7 @@ loser:
 }
 
 NSS_IMPLEMENT PRBool
-nssToken_IsPrivateKeyAvailable
-(
+nssToken_IsPrivateKeyAvailable (
   NSSToken *token,
   NSSCertificate *c,
   nssCryptokiObject *instance
